//=============================================================================
// LazyParser.NET - C# Expression Parser for .NET 2.0 
//
// Copyright (c) 2008 Philippe Leybaert
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//=============================================================================

using System;
using System.Collections.Generic;

namespace Activa.LazyParser
{
    public class AsExpression : Expression
    {
        private readonly Expression _objectExpression;
        private readonly Expression _typeExpression;

        public AsExpression(Expression objectExpression, Expression typeExpression)
        {
            _objectExpression = objectExpression;
            _typeExpression = typeExpression;
        }

        public override ValueExpression Evaluate(IParserContext context)
        {
            ClassName className = _typeExpression.Evaluate(context).Value as ClassName;

            if (className == null)
                throw new Exception("as operator requires type");

            Type checkType = className.Type;
            ValueExpression objectValue = _objectExpression.Evaluate(context);
            Type objectType = objectValue.Type;

            if (objectValue.Value == null)
                return Exp.Value(null, checkType);

            objectType = Nullable.GetUnderlyingType(objectType) ?? objectType;

            if (!objectType.IsValueType)
                return Exp.Value(objectValue.Value, checkType);

            if ((Nullable.GetUnderlyingType(checkType) ?? checkType) == objectType)
                return Exp.Value(objectValue.Value, checkType);

            return Exp.Value(null, checkType);
        }
    }
}
