//=============================================================================
// ProMesh.NET - .NET Web Application Framework 
//
// Copyright (c) 2003-2008 Philippe Leybaert
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//=============================================================================

using System;
using System.Collections.Generic;

namespace Activa.ProMesh
{
    [AttributeUsage(AttributeTargets.Method)]
    public class AjaxAttribute : Attribute
    {
        private readonly string _javaScriptAlias;
        private bool _useFormData = false;
        private bool _returnXml = false;

        public AjaxAttribute()
        {
        }

        public AjaxAttribute(string javascriptAlias)
        {
            _javaScriptAlias = javascriptAlias;
        }

        public AjaxAttribute(bool useFormData)
        {
            _useFormData = useFormData;
        }

        public AjaxAttribute(bool useFormData, string javascriptAlias)
        {
            _useFormData = useFormData;
            _javaScriptAlias = javascriptAlias;
        }

        public string JavaScriptAlias
        {
            get { return _javaScriptAlias; }
        }

        public bool UseFormData
        {
            get { return _useFormData; }
            set { _useFormData = value; }
        }

        public bool ReturnXml
        {
            get { return _returnXml; }
            set { _returnXml = value; }
        }
    }

    internal class AjaxOkAttribute : Attribute
    {
    }
    
}
