//=============================================================================
// ProMesh.NET - .NET Web Application Framework 
//
// Copyright (c) 2003-2008 Philippe Leybaert
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//=============================================================================

using System;
using System.Collections.Generic;

namespace Activa.ProMesh
{
    public class CheckBoxControl : Control
    {
        private bool _checked;
        private string _onClick;

        public CheckBoxControl(string name) : base(name)
        {
        }

        public bool Checked
        {
            get { return _checked; }
            set { _checked = value; }
        }

        public string OnClick
        {
            get { return _onClick; }
            set { _onClick = value; }
        }

        protected override string GenerateHtml(View view, string className, string classNameError)
        {
            string s = "<input type=\"checkbox\" value=\"1\"";

            s = AddIdAttribute(s);
            s = AddNameAttribute(s);
            s = AddClassAttribute(s, className, classNameError);
            s = AddEnabledAttribute(s);
            s = AddOnChangeAttribute(s);

            if (!string.IsNullOrEmpty(_onClick))
                s += " onclick=\"" + _onClick + "\"";

            if (Checked)
                s += " checked=\"checked\"";

            return s + " />";
        }

        protected override void HandlePostback(ClientDataCollection postData)
        {
            Checked = (postData.Get(Name) == "1");
        }

        public override object DataSource
        {
            get { return null; }
            set { }
        }

        public static string DefaultCssClass
        {
            set
            {
                SetDefaultCssClass<CheckBoxControl>(value, null);
            }
        }

        public static string DefaultCssClassError
        {
            set
            {
                SetDefaultCssClass<CheckBoxControl>(null, value);
            }
        }

    }
}
