//=============================================================================
// ProMesh.NET - .NET Web Application Framework 
//
// Copyright (c) 2003-2008 Philippe Leybaert
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//=============================================================================

using System;
using System.Collections.Generic;
using System.IO.Compression;
using System.Reflection;
using System.Text;
using System.Web;

namespace Activa.ProMesh
{
    internal class ResourceHttpHandler : IHttpHandler
    {
        public void ProcessRequest(HttpContext context)
        {
            string path = UrlHelper.GetUrlPathWithoutExtension(context.Request.AppRelativeCurrentExecutionFilePath, context.Request.PathInfo);

            if (path.StartsWith("_$res$_/"))
            {
                string[] parts = path.Substring(8).Split('/');

                SendResource(parts[1], parts[2], parts[0]);
            }
        }

        public bool IsReusable { get { return true; } }

        private static void SendResource(string assemblyName, string resourceKey, string contentType)
        {
            assemblyName = UrlHelper.DecodeFromURL(assemblyName);
            resourceKey = UrlHelper.DecodeFromURL(resourceKey);
            contentType = UrlHelper.DecodeFromURL(contentType);

            string eTag = BuildETag(assemblyName, resourceKey);

            if (WebAppContext.Request.CheckETag(eTag))
            {
                WebAppContext.Response.SetETag(eTag);
                WebAppContext.Response.StatusCode = 304;
            }
            else
            {
                HandleCompression();

                Assembly assembly = Assembly.Load(assemblyName);

                WebAppContext.Response.ContentType = contentType;
                WebAppContext.Response.SetETag(eTag);

                byte[] resourceData = ResourceHelper.GetResourceBytes(assembly, resourceKey);

                if (contentType.StartsWith("text/"))
                {
                    Encoding encoding;

                    resourceData = ResourceHelper.StripEncodingBOM(resourceData, out encoding);

                    WebAppContext.Response.ContentEncoding = encoding;
                }

                WebAppContext.Response.BinaryWrite(resourceData);    
            }
        }

        private static string BuildETag(string assemblyName, string resource)
        {
            return assemblyName.GetHashCode().ToString("X8") + resource.GetHashCode().ToString("X8") + ":" + _etagSignature;
        }

        private static readonly string _etagSignature = DateTime.Now.Ticks.ToString("X16");

        private static void HandleCompression()
        {
//            HttpContext httpContext = HttpContext.Current;
//            HttpResponse httpResponse = httpContext.Response;
//
//            string acceptedEncoding = (httpContext.Request.Headers["Accept-Encoding"] ?? "").ToLower();
//
//            if (acceptedEncoding.Contains("gzip"))
//            {
//                httpResponse.Filter = new GZipStream(httpResponse.Filter, CompressionMode.Compress);
//
//                httpResponse.AppendHeader("Content-Encoding", "gzip");
//            }
//            else
//            {
//                httpResponse.Filter = new DeflateStream(httpResponse.Filter,CompressionMode.Compress);
//
//                httpResponse.AppendHeader("Content-Encoding", "deflate");
//            }
        }
    }
}
