//=============================================================================
// ProMesh.NET - .NET Web Application Framework 
//
// Copyright (c) 2003-2008 Philippe Leybaert
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//=============================================================================

using System;
using System.Text.RegularExpressions;

namespace Activa.ProMesh
{
	public class RouteValidator
	{
		private readonly string _parameter;
		private readonly Regex _regex;
		private readonly RouteValidateResult _routeValidateResult;

		public RouteValidator(string parameter, string regex) : this(parameter, regex, RouteValidateResult.SkipOnFail)
		{
		}

		public RouteValidator(string parameter, string regex, RouteValidateResult routeValidateResult)
		{
			_parameter = parameter;

			_regex = new Regex("^" + regex + "$");

			_routeValidateResult = routeValidateResult;
		}

		public string Parameter
		{
			get { return _parameter; }
		}

		public RouteValidateResult RouteValidateResult
		{
			get { return _routeValidateResult; }
		}

		public bool Validate(string param)
		{
			return _regex.IsMatch(param);
		}
	}
}