//=============================================================================
// SharpTemplate.NET - Generic template parser
//
// Copyright (c) 2008 Philippe Leybaert
//
// Permission is hereby granted, free of charge, to any person obtaining a copy 
// of this software and associated documentation files (the "Software"), to deal 
// in the Software without restriction, including without limitation the rights 
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
// copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in 
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//=============================================================================

using System;
using System.Collections.Generic;

namespace Activa.SharpTemplate.Config
{
	/*
	 * Syntax:
	 *    <foreach var="varName" in="expression">
	 *      ...
	 *    </foreach>
	 * 
	 *    <if condition="conditionExpression">
	 *      ...
	 *    <elseif condition="conditionExpression" />
	 *      ...
	 *    <else>
	 *      ...
	 *    </if>
	 * 
	 *    $variable
	 * 
	 *    $variable.property
	 * 
	 *    ${expression}
	 *
     *    ${expression # formatstring}
	 */

	public class Xml : SharpTemplateConfig
    {
        public Xml()
        {
			AddToken(TemplateTokenType.ForEach, @"<foreach\s+var=(?<q>""|')(?<iterator>[a-z_][a-z0-9_]*)\k<q>\s+in=(?<q>""|')(?<expr>.*?)\k<q>\s*>", true);
            AddToken(TemplateTokenType.EndBlock, @"</foreach\s*>", true);
            AddToken(TemplateTokenType.EndBlock, @"</if\s*>", true);
			AddToken(TemplateTokenType.If, @"<if\s+condition=(?<q>""|')(?<expr>.*?)\k<q>\s*>", true);
			AddToken(TemplateTokenType.ElseIf, @"<elseif\s+condition=(?<q>""|')(?<expr>.*?)\k<q>\s*/>", true);
            AddToken(TemplateTokenType.Else, @"<else\s*/>", true);
            AddToken(TemplateTokenType.Expression, @"\${(?<expr>.*?)(\s*\#\s*(?<fmt>.*?)\s*)?}", false);
            AddToken(TemplateTokenType.Expression, @"\$(?<expr>[a-zA-Z_][a-zA-Z0-9_\.@]*?)(?![a-zA-Z0-9_\.@])", false);
        }

		private string ReplaceEscapes(string s)
		{
			return s.Replace("&lt;", "<").Replace("&gt;", ">").Replace("&amp;", "&").Replace("&quot;", "\"").Replace("&apos;", "'");
		}

		protected override System.Collections.IEnumerable OnEvalForeach(Activa.LazyParser.ExpressionParser parser, TokenMatch tokenMatch, Activa.LazyParser.IParserContext context)
		{
			tokenMatch.SubMatches["expr"] = ReplaceEscapes(tokenMatch.SubMatches["expr"]);

			return base.OnEvalForeach(parser, tokenMatch, context);
		}

		protected override bool OnEvalIf(Activa.LazyParser.ExpressionParser parser, TokenMatch tokenMatch, Activa.LazyParser.IParserContext context)
		{
			tokenMatch.SubMatches["expr"] = ReplaceEscapes(tokenMatch.SubMatches["expr"]);

			return base.OnEvalIf(parser, tokenMatch, context);
		}
    }
}
